# DADA2 Sequences to phyloseq object R script
#
# Kevin Lee, Klassen Lab, MCB, UConn, Storrs, CT
#
# Set the working directory

#setwd("~/klee/Projects/qc_analysis/test/")

# Load the required packages

library(dada2)
library(phyloseq)

# Set path to fastq files -- This needs to be customized to your workflow

path <- file.path(".")

# Read in file names, the pattern portion of this command needs to be consistent with the naming of your fastq files

fnFs <- sort(list.files(path, pattern = "_R1_001.fastq", full.names = TRUE))
fnRs <- sort(list.files(path, pattern = "_R2_001.fastq", full.names = TRUE))

# Assign sample names, as constructed, this command will give your samples names according to everything before the first use of underscore, if you have a different character you want to use for pattern matching, change this command accordingly

sample.names <- sapply(strsplit(basename(fnFs), "_"), '[', 1)

# Sanity check step, you can generate a composite of the quality profiles of your reads, similar to a fastQC report. This can be helpful later if you have results that don't look like what you expect. The numbers in brackets should be the number of samples you have.

f_qual <- plotQualityProfile(fnFs[1:48], aggregate = TRUE)
r_qual <- plotQualityProfile(fnRs[1:48], aggregate = TRUE)

jpeg(filename = "forward_quality_profile.jpg", width = 800, height = 600)
plot(f_qual)
dev.off()

jpeg(filename = "reverse_quality_profile.jpg", width = 800, height = 600)
plot(r_qual)
dev.off()

# Filtering and Trimming

# Setting paths for filtered files to be saved

filt_path <- file.path(path, "filtered")
filtFs <- file.path(filt_path, paste0(sample.names, "_F_filt.fastq.gz"))
filtRs <- file.path(filt_path, paste0(sample.names, "_R_filt.fastq.gz"))

# Filtering the reads: the truncLen parameter should be decided by looking at the quality profile of your reads. This can be done prior to running the script using the commands above. The maxEE parameter filters by expected error rates therefore higher numbers are less stringent than lower numbers. Multithread can be set to TRUE if being run in native R in a non-Windows environment (Linux or MacOS).

out <- filterAndTrim(fnFs, filtFs, fnRs, filtRs, truncLen = c(240, 200), trimLeft = c(0,20), maxN = 0, maxEE = c(2,2), truncQ = 2, rm.phix = TRUE, compress = TRUE, multithread = TRUE)

# Learning error profiles

errF <- learnErrors(filtFs, multithread = TRUE)
errR <- learnErrors(filtRs, multithread = TRUE)

# Sanity check: Plot error profiles

errF_plot <- plotErrors(errF, nominalQ = TRUE)
errR_plot <- plotErrors(errR, nominalQ = TRUE)

jpeg(filename = "F_error.jpg", width = 800, height = 600)
plot(errF_plot)
dev.off()

jpeg(filename = "R_error.jpg", width = 800, height = 600)
plot(errR_plot)
dev.off()

# Dereplicating samples for computational efficiency

derepFs <- derepFastq(filtFs, verbose = TRUE)
derepRs <- derepFastq(filtRs, verbose = TRUE)

names(derepFs) <- sample.names
names(derepRs) <- sample.names

# Inferring sequence variants

dadaFs <- dada(derepFs, err = errF, multithread = TRUE)
dadaRs <- dada(derepRs, err = errR, multithread = TRUE)

# Merging read pairs

mergers <- mergePairs(dadaFs, derepFs, dadaRs, derepRs, verbose = TRUE)

# Make the sequence table.

seqtab <- makeSequenceTable(mergers)

# Removing Chimeras. I am also including a sanity check step that prints the output of these steps to a pdf

seqtab.nochim <- removeBimeraDenovo(seqtab, method = "consensus", multithread = TRUE, verbose = TRUE)

seqtab_size <- dim(seqtab)
no.chim_size <- dim(seqtab.nochim)
kept <- sum(seqtab.nochim)/sum(seqtab)
cat("Dimensions of seqtab", seqtab_size, "Dimensions of seqtab.nochim", no.chim_size, "Percent of reads remaining after chimera removal", kept, file = "chimera_report.txt", fill = TRUE, sep = "\n")

# Tracking reads through the system, the last sanity check. Makes a csv that shows the read counts of each sample after each step of the process

getN <- function(x) sum(getUniques(x))
track <- cbind(out, sapply(dadaFs, getN), sapply(mergers, getN), rowSums(seqtab), rowSums(seqtab.nochim))
colnames(track) <- c("Input", "Filtered", "Denoised", "Merged", "Tabled", "Non-Chimeric")
write.csv(track, file = "Read_fates.csv", row.names = TRUE)

# Assigning taxonomy. You need to have the reference database file downloaded and make sure the path in the command agrees

taxa <- assignTaxonomy(seqtab.nochim, "silva_nr_v128_train_set.fa.gz", multithread = TRUE)

# Making a tree. Useful if you are going to do phylogeny based analyses downstream. Also handy as part of a complete phyloseq object, the added computational time may be worthwhile as opposed to backtracking and trying to do this as a separate step later. Requires DEIPHER and phangorn.

library(DECIPHER)
library(phangorn)

seqs <- getSequences(seqtab)
names(seqs) <- seqs
alignment <- AlignSeqs(DNAStringSet(seqs), anchor = NA)

phang.align <- phyDat(as(alignment, "matrix"), type = "DNA")
dm <- dist.ml(phang.align)
treeNJ <- NJ(dm)
fit <- pml(treeNJ, data = phang.align)
fitGTR <- update(fit, k=4, inv = 0.2)
fitGTR <- optim.pml(fitGTR, model = "GTR", optInv = TRUE, optGamma = TRUE, rearrangement = "stochastic", control = pml.control(trace =0))
detach("package:phangorn", unload = TRUE)
detach("package:DECIPHER", unload = TRUE)

# Building the phyloseq object. Any metadata needs to be read in before, as shown in the meta command.

meta <- read.csv("qc_meta.csv", header = TRUE, row.names = 1)

ps0 <- phyloseq(tax_table(taxa), otu_table(seqtab.nochim, taxa_are_rows = FALSE), sample_data(meta), phy_tree(fitGTR$tree))

library(ggplot2)
library(plyr)
library(dplyr)

# Determining and removing spurious ASV calls

taxa.print <- taxa
rownames(taxa.print) <- NULL

taxa.counts <- taxa_sums(ps0)

taxa.summary <- cbind(taxa.print, taxa.counts)
rownames(taxa.summary) <- NULL

write.csv(taxa.summary, file = "taxa_summary.csv")

spurious <- c("Streptococcus", "Capnocytophaga")

ps1 <- subset_taxa(ps0, !is.na(Genus) & !Genus %in% spurious)

ps1rab <- transform_sample_counts(ps1, function(x) {x/sum(x)})

ps1bar <- ps1rab %>% psmelt() %>% arrange(Genus)

library(viridis)

ps1bar_plot <- ggplot(ps1bar, aes(x = Sample, y = Abundance, fill = Genus)) + geom_bar(stat = "identity") + scale_fill_manual(values = c("Black", "Orange", "Sky Blue", "#009980", "Yellow", "Blue", "#CC6600", "#CC9983")) + theme(axis.title.x = element_blank(), axis.text.x = element_text(angle = 270, hjust = 0.5, vjust = 0.5), panel.background = element_blank())

jpeg(file = "qc_barplot.jpg")
plot(ps1bar_plot)
dev.off()

qc_wuf <- ordinate(ps1rab, method = "PCoA", distance = "wunifrac")

qc_wuf_plot <- plot_ordination(ps1rab, qc_wuf, color = "time", shape = "preservative") + geom_point(size = 3) + scale_color_discrete() + theme(panel.background = element_blank(), panel.border = element_rect(fill = "NA"))

jpeg(file = "qc_wuf_plot.jpg")
plot(qc_wuf_plot)
dev.off()

ps1rab_dist <- distance(ps1rab, method = "wunifrac")

ps1rab_sd <- data.frame(sample_data(ps1rab))

library(vegan)

qc_adonis_pres <- adonis(ps1rab_dist ~ preservative, ps1rab_sd)
qc_adonis_time <- adonis(ps1rab_dist ~ time, ps1rab_sd)
qc_adonis_presxtime <- adonis(ps1rab_dist ~ preservative * time, ps1rab_sd)
qc_adonis_timexpres <- adonis(ps1rab_dist ~ time * preservative, ps1rab_sd)

sink(file = "adonis_summary.txt", append = TRUE, type = "output")
qc_adonis_pres
qc_adonis_time
qc_adonis_presxtime
qc_adonis_timexpres
sink()

set.seed(100)

ps2 <- rarefy_even_depth(ps1)

ps2rab <- transform_sample_counts(ps2, function(x) {x/(sum(x))})

ps2bar <- ps2rab %>% psmelt() %>% arrange(Genus)

ps2bar_plot <- ggplot(ps2bar, aes(x = Sample, y = Abundance, fill = Genus)) + geom_bar(stat = "identity") + scale_fill_manual(values = c("Black", "Orange", "Sky Blue", "#009980", "Yellow", "Blue", "#CC6600", "#CC9983")) + theme(axis.title.x = element_blank(), axis.text.x = element_text(angle = 270, hjust = 0.5, vjust = 0.5), panel.background = element_blank())

ps2wuf <- ordinate(ps2, method = "PCoA", distance = "wunifrac")

ps2wuf_plot <- plot_ordination(ps1rab, qc_wuf, color = "time", shape = "preservative") + geom_point(size = 3) + scale_color_discrete() + theme(panel.background = element_blank(), panel.border = element_rect(fill = "NA")) + ggtitle("Weighted UniFrac of subsampled QC communities")

qc_hm <- read.csv("qc_heatmap.csv", header = TRUE)

ggplot(qc_hm_log, aes(preservative, taxon)) + geom_tile(aes(fill = value)) + scale_fill_gradient2(breaks = c(3, 2.5, 2, 1.5, 1, 0.5, 0, -0.5, -1, -1.5, -2, -2.5, -3), low = "Blue",mid = "black", high = "Yellow", guide = "colorbar") + coord_equal()

